Formatting

Struct Formatting 

pub struct Formatting {
    pub row_spacing: u8,
    pub tile_spacing: u8,
    pub top_indent: u8,
    pub left_indent: u8,
    pub bottom_indent: u8,
}
Formatting represents instructions for TileMap<T> on how to draw tilemap to the terminal

row_spacing - number of additional newlines between every row, defaults to 1 tile_spacing - number of spaces between every tile, defaults to 1 top_indent - number of newlines to insert before drawing the tilemap, defaults to 3 left_indent - number of tabs to insert at the start of every row, defaults to 1 bottom_indent - number of newlines to insert after drawing the tilemap, defaults to 2

Examples

use cli_tilemap::Formatting;

let f = Formatting::default();
assert_eq!(f.row_spacing, 1);
assert_eq!(f.tile_spacing, 1);
assert_eq!(f.top_indent, 3);
assert_eq!(f.left_indent, 1);
assert_eq!(f.bottom_indent, 2);

Fields

row_spacing: u8tile_spacing: u8top_indent: u8left_indent: u8bottom_indent: u8